@@ -113,6 +113,7 @@ static char *s_chtml30_end_menu_tag       (void *pdoc, Node *node);
 static char *s_chtml30_start_plaintext_tag       (void *pdoc, Node *node);
 static char *s_chtml30_start_plaintext_tag_inner (void *pdoc, Node *node);
 static char *s_chtml30_end_plaintext_tag         (void *pdoc, Node *node);
+static char *s_chtml30_newline_mark       (void *pdoc, Node *node);
 
 static void  s_init_chtml30(chtml30_t *chtml, Doc *doc, request_rec *r, device_table *spec);
 
@@ -385,6 +386,11 @@ tag_handler chtml30_handler[] = {
     s_chtml30_start_marquee_tag,
     s_chtml30_end_marquee_tag,
   },
+  /* tagNLMARK */
+  {
+    s_chtml30_newline_mark,
+    NULL,
+  },
 };
 
 
@@ -581,7 +587,6 @@ s_chtml30_start_html_tag(void *pdoc, Node *UNUSED(node))
   /* start HTML tag                                                           */
   /*--------------------------------------------------------------------------*/
   W_L("<html>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -602,7 +607,6 @@ s_chtml30_end_html_tag(void *pdoc, Node *UNUSED(child))
   Doc         *doc     = chtml30->doc;
 
   W_L("</html>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -705,7 +709,6 @@ s_chtml30_start_meta_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -742,7 +745,6 @@ s_chtml30_start_head_tag(void* pdoc, Node* UNUSED(node))
   Doc         *doc     = chtml30->doc;
 
   W_L("<head>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -763,7 +765,6 @@ s_chtml30_end_head_tag(void *pdoc, Node *UNUSED(node))
   Doc       *doc     = chtml30->doc;
 
   W_L("</head>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -784,7 +785,6 @@ s_chtml30_start_title_tag(void *pdoc, Node *UNUSED(node))
   Doc           *doc     = chtml30->doc;
 
   W_L("<title>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -805,7 +805,6 @@ s_chtml30_end_title_tag(void *pdoc, Node *UNUSED(child))
   Doc          *doc     = chtml30->doc;
 
   W_L("</title>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -847,7 +846,6 @@ s_chtml30_start_base_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -936,7 +934,6 @@ s_chtml30_start_body_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -959,7 +956,6 @@ s_chtml30_end_body_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml30->doc;
 
   W_L("</body>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -1107,7 +1103,6 @@ s_chtml30_end_a_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml30->doc;
 
   W_L("</a>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -1150,7 +1145,6 @@ s_chtml30_start_br_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -1209,7 +1203,6 @@ s_chtml30_end_tr_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("<br>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -1288,7 +1281,6 @@ s_chtml30_end_font_tag(void *pdoc, Node *UNUSED(child))
     W_L("</font>");
     chtml30->font_flag--;
   }
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -1350,7 +1342,6 @@ s_chtml30_start_form_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -1373,7 +1364,6 @@ s_chtml30_end_form_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml30->doc;
 
   W_L("</form>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -1444,7 +1434,7 @@ s_chtml30_start_input_tag(void *pdoc, Node *node)
   }
   if (value && *value) {
     W_L(" value=\"");
-    W_V(value);
+    W_V(chxj_add_slash_to_doublequote(doc->pool, value));
     W_L("\"");
   }
   if (accesskey && *accesskey) {
@@ -1483,7 +1473,6 @@ s_chtml30_start_input_tag(void *pdoc, Node *node)
     W_L(" checked");
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -1523,7 +1512,6 @@ s_chtml30_start_center_tag(void *pdoc, Node *UNUSED(node))
   doc     = chtml30->doc;
 
   W_L("<center>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -1547,7 +1535,6 @@ s_chtml30_end_center_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml30->doc;
 
   W_L("</center>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -1624,7 +1611,6 @@ s_chtml30_start_hr_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -1785,7 +1771,6 @@ s_chtml30_start_img_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -1865,7 +1850,6 @@ s_chtml30_start_select_tag(void *pdoc, Node *child)
     W_L(" multiple");
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -1885,7 +1869,6 @@ s_chtml30_end_select_tag(void *pdoc, Node *UNUSED(child))
   Doc          *doc   = chtml30->doc;
 
   W_L("</select>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -2008,7 +1991,6 @@ s_chtml30_start_div_tag(void *pdoc, Node *child)
     W_L("\"");
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -2028,7 +2010,6 @@ s_chtml30_end_div_tag(void *pdoc, Node *UNUSED(node))
   Doc          *doc     = chtml30->doc;
 
   W_L("</div>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2049,7 +2030,6 @@ s_chtml30_start_ul_tag(void *pdoc, Node *UNUSED(node))
   Doc         *doc     = chtml30->doc;
 
   W_L("<ul>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2070,7 +2050,6 @@ s_chtml30_end_ul_tag(void *pdoc, Node *UNUSED(child))
   Doc           *doc     = chtml30->doc;
 
   W_L("</ul>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2092,7 +2071,6 @@ s_chtml30_start_pre_tag(void *pdoc, Node *UNUSED(node))
 
   chtml30->pre_flag++;
   W_L("<pre>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2113,7 +2091,6 @@ s_chtml30_end_pre_tag(void *pdoc, Node *UNUSED(child))
   Doc           *doc     = chtml30->doc;
 
   W_L("</pre>");
-  W_NLCODE();
   chtml30->pre_flag--;
 
   return chtml30->out;
@@ -2163,7 +2140,6 @@ s_chtml30_start_p_tag(void *pdoc, Node *node)
     W_L("\"");
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -2186,7 +2162,6 @@ s_chtml30_end_p_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml30->doc;
 
   W_L("</p>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2233,7 +2208,6 @@ s_chtml30_start_ol_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2254,7 +2228,6 @@ s_chtml30_end_ol_tag(void *pdoc, Node *UNUSED(node))
   Doc       *doc     = chtml30->doc;
 
   W_L("</ol>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2376,7 +2349,6 @@ s_chtml30_end_h1_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml30->doc;
 
   W_L("</h1>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2437,7 +2409,6 @@ s_chtml30_end_h2_tag(void *pdoc, Node *UNUSED(child))
   Doc         *doc     = chtml30->doc;
 
   W_L("</h2>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2499,7 +2470,6 @@ s_chtml30_end_h3_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc   = chtml30->doc;
 
   W_L("</h3>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2560,7 +2530,6 @@ s_chtml30_end_h4_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc     = chtml30->doc;
 
   W_L("</h4>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2621,7 +2590,6 @@ s_chtml30_end_h5_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc     = chtml30->doc;
 
   W_L("</h5>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2682,7 +2650,6 @@ s_chtml30_end_h6_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc     = chtml30->doc;
 
   W_L("</h6>");
-  W_NLCODE();
 
   return chtml30->out;
 }
@@ -2743,7 +2710,6 @@ s_chtml30_start_textarea_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -2763,7 +2729,6 @@ s_chtml30_end_textarea_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc     = chtml30->doc;
 
   W_L("</textarea>");
-  W_NLCODE();
   chtml30->textarea_flag--;
 
   return chtml30->out;
@@ -2871,7 +2836,6 @@ s_chtml30_start_blockquote_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc     = chtml30->doc;
   W_L("<blockquote>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -2890,7 +2854,6 @@ s_chtml30_end_blockquote_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc     = chtml30->doc;
   W_L("</blockquote>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -2909,7 +2872,6 @@ s_chtml30_start_dir_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc     = chtml30->doc;
   W_L("<dir>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -2928,7 +2890,6 @@ s_chtml30_end_dir_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc = chtml30->doc;
   W_L("</dir>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -2947,7 +2908,6 @@ s_chtml30_start_dl_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc     = chtml30->doc;
   W_L("<dl>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -2966,7 +2926,6 @@ s_chtml30_end_dl_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc     = chtml30->doc;
   W_L("</dl>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -3107,7 +3066,6 @@ s_chtml30_end_marquee_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc = chtml30->doc;
   W_L("</marquee>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -3144,7 +3102,6 @@ s_chtml30_end_blink_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc = chtml30->doc;
   W_L("</blink>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -3163,7 +3120,6 @@ s_chtml30_start_menu_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc     = chtml30->doc;
   W_L("<menu>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -3182,7 +3138,6 @@ s_chtml30_end_menu_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   Doc       *doc = chtml30->doc;
   W_L("</menu>");
-  W_NLCODE();
   return chtml30->out;
 }
 
@@ -3235,6 +3190,19 @@ s_chtml30_end_plaintext_tag(void *pdoc, Node *UNUSED(child))
   chtml30_t *chtml30 = GET_CHTML30(pdoc);
   return chtml30->out;
 }
+
+
+/**
+ *  * It is handler who processes the New Line Code.
+ *   */
+static char *
+s_chtml30_newline_mark(void *pdoc, Node *UNUSED(node))
+{
+  chtml30_t *chtml30 = GET_CHTML30(pdoc);
+  Doc *doc = chtml30->doc;
+  W_NLCODE();
+  return chtml30->out;
+}
 /*
  * vim:ts=2 et
  */
