@@ -157,10 +157,14 @@ qs_alloc_zero_byte_string(apr_pool_t *pool)
 char *
 qs_trim_string(apr_pool_t *p, char *s)
 {
-  char *ss = apr_pstrdup(p, s);
-  int len = strlen(s);
+  char *ss;
+  int len;
   int ii;
 
+  if (! s) return apr_pstrdup(p, "");
+
+  ss = apr_pstrdup(p, s);
+  len = strlen(s);
   ii = 0;
   for (ii = 0;is_white_space(*ss) && ii < len; ss++, ii++);
 
